# encoding: UTF-8
class Document < ActiveRecord::Base
  has_paper_trail
  include Uuidextension
  has_many :transfers, :through => :attachements
  has_many :attachements, :dependent => :destroy

  @@document_path = "#{Rails.root}/public/documents/"
  
  after_destroy { |doc| 
    `rm #{doc.absolute_path} #{doc.thumbnail_path}`
  }

  def self.document_directory_path
    Rails.public_path+'/documents/'
  end
  def self.document_directory_url
    "/documents/"
  end
  def icon_path
    case filename[-4..-1].downcase
      when 'jpeg','.jpg'
        '/images/icon_jpeg.gif'
      when '.pdf'
        '/images/icon_pdf.gif'
      when '.txt','.doc','docx'
        '/images/icon_text.gif'
      when '.odt'
        '/images/icon_odt.png'
      when '.ods'
        '/images/icon_ods.png'
      else
        '/images/icon_unknown.gif'
    end
  end
  def absolute_path
    Document.document_directory_path+filename
  end
  def absolute_trash_path
    Document.document_directory_path+'del~'+filename
  end
  def trashed?
    filename =~ /^del~/
  end
  def detached?
    attachements.count == 0
  end
  def absolute_url
    '/documents/'+filename
  end
  def url
    SITE_URL.gsub(/\/$/,'')+absolute_url
  end
  def self.get_unused_filename(datum,extension)
    upload_path = @@document_path
    filename_base = datum

    # vymyslime vhodne jmeno pro soubor
    iterator = ''
    while  File.exist?(upload_path + filename_base + iterator.to_s + extension) or (!Document.find_by_filename(filename_base + iterator.to_s + extension).nil?)
      if (iterator == '' )
        iterator = 0
        filename_base += '_'
      end
      iterator = iterator + 1
    end
    return filename_base + iterator.to_s + extension
  end
  def self.create_from_download(file,filenamebase)
    log = Logger.new(File.open(Rails.root.to_s+'/log/debug.log','a'))
    if not( file.nil? ) and not (file.is_a? String)
      file.tempfile.binmode
      
      # najdi a zkontroluj priponu
      if file.original_filename =~ /(pdf|xls|ods|jpe?g)$/i
        ext = ($~[1]).downcase
      else
        return nil
      end

      upload_path = "#{Rails.root}/public/documents/"
      tmp_path = '/tmp/'

      if ext =~ /jpe?g/i
        # zapis soubor
        tmp_filename = tmp_path + Document.get_unused_filename('ucetupload','.jpeg')  #TODO: nahradit tmp_file = Tempfile.new('ucetupload','.jpeg'); tmp_filename = tmp_file.path
        File.open(tmp_filename, "wb") do |f|
          f.write(file.read)
        end

        # convert jpg->pdf
        unused_filename_base = Document.get_unused_filename(filenamebase,'.pdf')
        filename = upload_path + unused_filename_base
        log.debug 'Conversion>' + `convert #{tmp_filename} #{filename}; rm #{tmp_filename};`
      else
        # zapis soubor
        unused_filename_base = Document.get_unused_filename(filenamebase,'.'+ext)
        filename = upload_path + unused_filename_base
        log.debug 'writing file>'+ filename
        File.open(filename, "w+b") do |f|
          f.write(file.read)
        end
      end 

      # create table entry
      document = Document.new(:filename=> unused_filename_base, :original_filename => file.original_filename)
      document.save
      return document
    else
      return nil # nebylo co prikladat, takze to neslo
    end
  end
  def filename_with_extension(ext)
    oldextension = File.extname(filename)
    ext = '.'+ext unless ext.match(/^\./)
    return filename.gsub(/#{oldextension}$/,ext)
  end
  def create_or_refresh_thumbnail
    log = Logger.new(File.open(Rails.root.to_s+'/log/debug.log','a'))
    log.debug 'Conversion>' + `convert #{absolute_path}[0] -thumbnail '210x297' -unsharp 0x0.5  #{thumbnail_path}`
  end
  def create_unless_exists_thumbnail
    create_or_refresh_thumbnail unless File.exists?(thumbnail_path)
  end
  def thumbnail_path; @@document_path+'thumbnails/'+filename_with_extension('jpeg') end
  def thumbnail_url
    thumbnails_url = Document.document_directory_url + 'thumbnails/'
    thumbnails_url+filename_with_extension('jpeg')
  end  
  def jpeg_path; @@document_path+'jpegs/'+filename_with_extension('jpeg') end
  def self.create_from_orphaned(filename,newfilename=nil)
    if File.exists?(document_directory_path + filename)
      `mv #{document_directory_path + filename} #{document_directory_path + newfilename}` unless newfilename.nil?
      document = Document.new(:filename=> newfilename.nil? ? filename : newfilename )
      document.save
      return document
      # TODO melo by to projit stejnym procesem jako pri create_from_download
    else
      return nil
    end    
  end
  def self.orphaned_on_disk
    return Dir.entries(document_directory_path).reject{ |file|
      (File.directory?(file)) or (file[0,1]==".") or (file[0,4]=='del~')
    }.select{ |filename|
      Document.find_by_filename(filename).nil?
     # /P1DD.*/.match(filename)
    }
  end
  def self.orphaned_in_database
    Document.all.select { |document|
      not(document.file_exists_on_disk?)
    }
  end
  def self.trashed
    return Dir.entries(document_directory_path).select{ |file|
      file[0,4]=='del~'
    }.select{ |filename|
      Document.find_by_filename(filename).nil?
    }
  end
  def file_exists_on_disk?
    File.exists?(Document.document_directory_path + filename)    
  end
  def to_s
    filename.to_s
  end
  def extension
    filename =~ /\.(.*)$/
    $~[1]
  end
  def basefilename
    filename.gsub(/\.(.*)$/,'')
  end
  def guessed_transfer_seq
    basefilename.gsub(/[a-z]$/,'')
  end
  def destroy_if_detached
    if detached?
      destroy
    end
  end
end





